@@ -33,6 +33,8 @@
 #include <libxfce4ui/libxfce4ui.h>
 #include <libxfce4panel/libxfce4panel.h>
 
+#include "monitor-label.h"
+
 
 #define BORDER 8
 
@@ -82,7 +84,6 @@ typedef struct
 
 typedef struct
 {
-    GtkWidget  *box;
     GtkWidget  *label;
     GtkWidget  *rcv_label;
     GtkWidget  *sent_label;
@@ -246,17 +247,18 @@ static gboolean update_monitors(t_global_monitor *global)
     {
         char* ip = get_ip_address(&(global->monitor->data));
         g_snprintf(caption, sizeof(caption), 
-                _("<< %s >> (%s)\nAverage of last %d measures:\n"
-                    "Incoming: %s/s\nOutgoing: %s/s\nTotal: %s/s"),
+                   _("<< %s >> (%s)\nAverage of last %d measures\n"
+                     "with an interval of %.2fs:\n"
+                     "Incoming: %s\nOutgoing: %s\nTotal: %s"),
                     get_name(&(global->monitor->data)), ip ? ip : _("no IP address"),
-                    HISTSIZE_CALCULATE, buffer[IN], buffer[OUT], buffer[TOT]);
+                    HISTSIZE_CALCULATE, global->monitor->options.update_interval / 1000.0,
+                    buffer[IN], buffer[OUT], buffer[TOT]);
         gtk_label_set_text(GTK_LABEL(global->tooltip_text), caption);
 
         if (global->monitor->options.show_values)
         {
             g_snprintf(received, sizeof(received), "%s", buffer_panel[IN]);
             gtk_label_set_text(GTK_LABEL(global->monitor->rcv_label), received);
-
             g_snprintf(sent, sizeof(sent), "%s", buffer_panel[OUT]);
             gtk_label_set_text(GTK_LABEL(global->monitor->sent_label), sent);
         }
@@ -284,108 +286,95 @@ static void run_update (t_global_monitor *global)
 
 
 /* ---------------------------------------------------------------------------------------------- */
-static void monitor_set_orientation (XfcePanelPlugin *plugin, GtkOrientation orientation, 
-                                     t_global_monitor *global)
+static gboolean monitor_set_size(XfcePanelPlugin *plugin, int size, t_global_monitor *global)
 {
     gint i;
+    XfcePanelPluginMode mode = xfce_panel_plugin_get_mode (plugin);
+
+    PRINT_DBG("monitor_set_size");
+
+    if (mode == XFCE_PANEL_PLUGIN_MODE_DESKBAR)
+    {
+        for (i = 0; i < SUM; i++)
+            gtk_widget_set_size_request(GTK_WIDGET(global->monitor->status[i]), BORDER, BORDER);
+        gtk_widget_set_size_request(GTK_WIDGET(plugin), size, -1);
+    }
+    else if (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL)
+    {
+        for (i = 0; i < SUM; i++)
+            gtk_widget_set_size_request(GTK_WIDGET(global->monitor->status[i]), -1, BORDER);
+        gtk_widget_set_size_request(GTK_WIDGET(plugin), size, -1);
+    }
+    else /* mode == XFCE_PANEL_PLUGIN_MODE_HORIZONTAL */
+    {
+        for (i = 0; i < SUM; i++)
+            gtk_widget_set_size_request(GTK_WIDGET(global->monitor->status[i]), BORDER, -1);
+        gtk_widget_set_size_request(GTK_WIDGET(plugin), -1, size);
+    }
+
+    xnlp_monitor_label_reinit_size_request(XNLP_MONITOR_LABEL(global->monitor->rcv_label));
+    xnlp_monitor_label_reinit_size_request(XNLP_MONITOR_LABEL(global->monitor->sent_label));
+    gtk_container_set_border_width(GTK_CONTAINER(global->box), size > 26 ? 2 : 1);
+
+    return TRUE;
+}
+
+
+/* ---------------------------------------------------------------------------------------------- */
+static void monitor_set_mode (XfcePanelPlugin *plugin, XfcePanelPluginMode mode,
+                              t_global_monitor *global)
+{
+    gint i;
+
+    PRINT_DBG("monitor_set_mode");
 
     if (global->timeout_id)
     {
         g_source_remove(global->timeout_id);
     }
 
-    gtk_widget_hide(GTK_WIDGET(global->ebox));
-    if (global->box)
+    if (mode == XFCE_PANEL_PLUGIN_MODE_DESKBAR)
     {
-        gtk_container_remove(GTK_CONTAINER(global->ebox), GTK_WIDGET(global->box));
+        xfce_hvbox_set_orientation(XFCE_HVBOX(global->box), GTK_ORIENTATION_VERTICAL);
+        gtk_label_set_angle(GTK_LABEL(global->monitor->label), 0);
+        gtk_misc_set_alignment(GTK_MISC(global->monitor->rcv_label), 0.5f, 1.0f);
+        gtk_misc_set_alignment(GTK_MISC(global->monitor->sent_label), 0.5f, 0.0f);
+        gtk_label_set_angle(GTK_LABEL(global->monitor->rcv_label), 0);
+        gtk_label_set_angle(GTK_LABEL(global->monitor->sent_label), 0);
+        for (i = 0; i < SUM; i++)
+            gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(global->monitor->status[i]),
+                                             GTK_PROGRESS_LEFT_TO_RIGHT);
     }
-    if (orientation == GTK_ORIENTATION_HORIZONTAL)
+    else if (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL)
     {
-        global->box = gtk_hbox_new(FALSE, 0);
+        xfce_hvbox_set_orientation(XFCE_HVBOX(global->box), GTK_ORIENTATION_VERTICAL);
+        gtk_label_set_angle(GTK_LABEL(global->monitor->label), 270);
+        gtk_misc_set_alignment(GTK_MISC(global->monitor->rcv_label), 0.5f, 1.0f);
+        gtk_misc_set_alignment(GTK_MISC(global->monitor->sent_label), 0.5f, 0.0f);
+        gtk_label_set_angle(GTK_LABEL(global->monitor->rcv_label), 270);
+        gtk_label_set_angle(GTK_LABEL(global->monitor->sent_label), 270);
+        for (i = 0; i < SUM; i++)
+        {
+            gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(global->monitor->status[i]),
+                    GTK_PROGRESS_LEFT_TO_RIGHT);
+        }
     }
-    else
+    else /* mode == XFCE_PANEL_PLUGIN_MODE_HORIZONTAL */
     {
-        global->box = gtk_vbox_new(FALSE, 0);
-    }
-    gtk_widget_show(global->box);
-
-    global->monitor->label = gtk_label_new(global->monitor->options.label_text);
-    gtk_widget_show(global->monitor->label);
-
-    global->monitor->rcv_label = gtk_label_new("");
-    gtk_label_set_width_chars(GTK_LABEL(global->monitor->rcv_label), 7);
-    gtk_misc_set_alignment(GTK_MISC(global->monitor->rcv_label), 1.0f, 0.5f);
-    gtk_widget_show(global->monitor->rcv_label);
-
-    for (i = 0; i < SUM; i++)
-    {
-        global->monitor->status[i] = GTK_WIDGET(gtk_progress_bar_new());
-    }
-
-    global->monitor->sent_label = gtk_label_new("");
-    gtk_label_set_width_chars(GTK_LABEL(global->monitor->sent_label), 7);
-    gtk_widget_show(global->monitor->sent_label);
-
-    if (orientation == GTK_ORIENTATION_HORIZONTAL)
-    {
-        global->monitor->box = GTK_WIDGET(gtk_hbox_new(FALSE, 0));
+        xfce_hvbox_set_orientation(XFCE_HVBOX(global->box), GTK_ORIENTATION_HORIZONTAL);
+        gtk_label_set_angle(GTK_LABEL(global->monitor->label), 0);
+        gtk_misc_set_alignment(GTK_MISC(global->monitor->rcv_label), 1.0f, 0.5f);
+        gtk_misc_set_alignment(GTK_MISC(global->monitor->sent_label), 0.0f, 0.5f);
+        gtk_label_set_angle(GTK_LABEL(global->monitor->rcv_label), 0);
+        gtk_label_set_angle(GTK_LABEL(global->monitor->sent_label), 0);
         for (i = 0; i < SUM; i++)
         {
             gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(global->monitor->status[i]),
                     GTK_PROGRESS_BOTTOM_TO_TOP);
         }
-        gtk_misc_set_alignment(GTK_MISC(global->monitor->sent_label), 0.0f, 0.5f);
-    }
-    else
-    {
-        global->monitor->box = GTK_WIDGET(gtk_vbox_new(FALSE, 0));
-        for (i = 0; i < SUM; i++)
-        {
-            gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(global->monitor->status[i]), 
-                    GTK_PROGRESS_LEFT_TO_RIGHT);
-        }
-        gtk_misc_set_alignment(GTK_MISC(global->monitor->sent_label), 1.0f, 0.5f);
     }
 
-    gtk_box_pack_start(GTK_BOX(global->monitor->box),
-                       GTK_WIDGET(global->monitor->label),
-                       TRUE, FALSE, BORDER / 2);
-
-    gtk_box_pack_start(GTK_BOX(global->monitor->box),
-                       GTK_WIDGET(global->monitor->rcv_label),
-                       TRUE, FALSE, BORDER / 2);
-
-    gtk_container_set_border_width(GTK_CONTAINER(global->monitor->box), BORDER / 2);
-    gtk_widget_show(GTK_WIDGET(global->monitor->box));
-
-    for (i = 0; i < SUM; i++)
-    {
-        gtk_widget_modify_bg(GTK_WIDGET(global->monitor->status[i]),
-                             GTK_STATE_PRELIGHT,
-                             &global->monitor->options.color[i]);
-        gtk_widget_modify_bg(GTK_WIDGET(global->monitor->status[i]),
-                             GTK_STATE_SELECTED,
-                             &global->monitor->options.color[i]);
-        gtk_widget_modify_base(GTK_WIDGET(global->monitor->status[i]),
-                               GTK_STATE_SELECTED,
-                               &global->monitor->options.color[i]);
-
-        gtk_widget_show(GTK_WIDGET(global->monitor->status[i]));
-
-        gtk_box_pack_start(GTK_BOX(global->monitor->box),
-                GTK_WIDGET(global->monitor->status[i]), FALSE, FALSE, 0);
-    }
-
-    gtk_box_pack_start(GTK_BOX(global->monitor->box),
-                     GTK_WIDGET(global->monitor->sent_label),
-                     TRUE, FALSE, BORDER / 2);
-
-    gtk_box_pack_start(GTK_BOX(global->box),
-                       GTK_WIDGET(global->monitor->box), FALSE, FALSE, 0);
-
-
-    gtk_container_add(GTK_CONTAINER(global->ebox), GTK_WIDGET(global->box));
-    gtk_widget_show(GTK_WIDGET(global->ebox));
+    monitor_set_size(plugin, xfce_panel_plugin_get_size(plugin), global);
 
     run_update( global );
 }
@@ -427,10 +416,11 @@ static t_global_monitor * monitor_new(XfcePanelPlugin *plugin)
     global = g_new(t_global_monitor, 1);
     global->timeout_id = 0;
     global->ebox = gtk_event_box_new();
+    gtk_event_box_set_visible_window(GTK_EVENT_BOX(global->ebox), FALSE);
+    gtk_event_box_set_above_child(GTK_EVENT_BOX(global->ebox), TRUE);
     gtk_widget_set_has_tooltip(global->ebox, TRUE);
     g_signal_connect(global->ebox, "query-tooltip", G_CALLBACK(tooltip_cb), global);
     gtk_widget_show(global->ebox);
-    global->box = NULL;
 
     global->tooltip_text = gtk_label_new(NULL);
     g_object_ref(global->tooltip_text);
@@ -456,13 +446,45 @@ static t_global_monitor * monitor_new(XfcePanelPlugin *plugin)
         global->monitor->history[i][1] = 0;
         global->monitor->history[i][2] = 0;
         global->monitor->history[i][3] = 0;
+
         global->monitor->net_max[i]    = INIT_MAX;
         
         global->monitor->options.max[i] = INIT_MAX;
     }
 
-    monitor_set_orientation (plugin, xfce_panel_plugin_get_orientation (plugin), global);
-    
+    /* Create widget containers */
+    global->box = xfce_hvbox_new(GTK_ORIENTATION_HORIZONTAL, FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(global->box), 2);
+    gtk_widget_show(GTK_WIDGET(global->box));
+
+    /* Create the title label */
+    global->monitor->label = gtk_label_new(global->monitor->options.label_text);
+    gtk_box_pack_start(GTK_BOX(global->box),
+                       GTK_WIDGET(global->monitor->label),
+                       TRUE, FALSE, 2);
+
+    /* Create sent and received labels */
+    global->monitor->rcv_label = xnlp_monitor_label_new("-");
+    global->monitor->sent_label = xnlp_monitor_label_new("-");
+    gtk_box_pack_start(GTK_BOX(global->box),
+                       GTK_WIDGET(global->monitor->rcv_label),
+                       TRUE, FALSE, 2);
+
+    /* Create the progress bars */
+    for (i = 0; i < SUM; i++)
+    {
+        global->monitor->status[i] = GTK_WIDGET(gtk_progress_bar_new());
+        gtk_box_pack_start(GTK_BOX(global->box),
+                           GTK_WIDGET(global->monitor->status[i]), FALSE, FALSE, 0);
+    }
+
+    /* Append sent label after the progress bars */
+    gtk_box_pack_start(GTK_BOX(global->box),
+                       GTK_WIDGET(global->monitor->sent_label),
+                       TRUE, FALSE, 2);
+
+    gtk_container_add(GTK_CONTAINER(global->ebox), GTK_WIDGET(global->box));
+
     return global;
 }
 
@@ -472,58 +494,70 @@ static void setup_monitor(t_global_monitor *global, gboolean supress_warnings)
 {
     gint i;
 
-    gtk_widget_hide(GTK_WIDGET(global->monitor->box));
-    gtk_widget_hide(global->monitor->label);
-    gtk_label_set_text(GTK_LABEL(global->monitor->label),
-            global->monitor->options.label_text);
-    gtk_widget_hide(global->monitor->rcv_label);
-    for (i = 0; i < SUM; i++)
-      gtk_widget_hide(global->monitor->status[i]);
-    gtk_widget_hide(global->monitor->sent_label);
+    if (global->timeout_id)
+        g_source_remove(global->timeout_id);
 
-    if (global->monitor->options.show_bars)
+    /* Show title label? */
+    if (global->monitor->options.use_label)
     {
-      for (i = 0; i < SUM; i++)
-      {
-        gtk_widget_show(global->monitor->status[i]);
+        gtk_label_set_text(GTK_LABEL(global->monitor->label), global->monitor->options.label_text);
+        gtk_widget_show(global->monitor->label);
+    }
+    else
+        gtk_widget_hide(global->monitor->label);
 
-        gtk_widget_modify_bg(GTK_WIDGET(global->monitor->status[i]),
-                             GTK_STATE_PRELIGHT,
-                             &global->monitor->options.color[i]);
-        gtk_widget_modify_bg(GTK_WIDGET(global->monitor->status[i]),
-                             GTK_STATE_SELECTED,
-                             &global->monitor->options.color[i]);
-        gtk_widget_modify_base(GTK_WIDGET(global->monitor->status[i]),
-                               GTK_STATE_SELECTED,
-                               &global->monitor->options.color[i]);
-        
-        /* Maximum */
-        if( global->monitor->options.auto_max )
+    /* Show values? */
+    if (global->monitor->options.show_values)
+    {
+        gtk_widget_show(global->monitor->rcv_label);
+        gtk_widget_show(global->monitor->sent_label);
+    }
+    else
+    {
+        gtk_widget_hide(global->monitor->rcv_label);
+        gtk_widget_hide(global->monitor->sent_label);
+    }
+
+    if (global->monitor->options.colorize_values)
+    {
+        gtk_widget_modify_fg(global->monitor->rcv_label, GTK_STATE_NORMAL,
+                             &global->monitor->options.color[IN]);
+        gtk_widget_modify_fg(global->monitor->sent_label, GTK_STATE_NORMAL,
+                             &global->monitor->options.color[OUT]);
+    }
+    else
+    {
+        gtk_widget_modify_fg(global->monitor->rcv_label, GTK_STATE_NORMAL, NULL);
+        gtk_widget_modify_fg(global->monitor->sent_label, GTK_STATE_NORMAL, NULL);
+    }
+
+    /* Setup the progress bars */
+    for (i = 0; i < SUM; i++)
+    {
+        if (global->monitor->options.show_bars)
         {
-            global->monitor->net_max[i] = INIT_MAX;
+            /* Automatic or fixed maximum */
+            if (global->monitor->options.auto_max)
+                global->monitor->net_max[i] = INIT_MAX;
+            else
+                global->monitor->net_max[i] = global->monitor->options.max[i];
+            gtk_widget_show(global->monitor->status[i]);
+
+            /* Set bar colors */
+            gtk_widget_modify_bg(GTK_WIDGET(global->monitor->status[i]),
+                                 GTK_STATE_PRELIGHT,
+                                 &global->monitor->options.color[i]);
+            gtk_widget_modify_bg(GTK_WIDGET(global->monitor->status[i]),
+                                 GTK_STATE_SELECTED,
+                                 &global->monitor->options.color[i]);
+            gtk_widget_modify_base(GTK_WIDGET(global->monitor->status[i]),
+                                   GTK_STATE_SELECTED,
+                                   &global->monitor->options.color[i]);
         }
         else
         {
-            global->monitor->net_max[i] = global->monitor->options.max[i];
+            gtk_widget_hide(global->monitor->status[i]);
         }
-      }
-    }
-
-    gtk_widget_show(GTK_WIDGET(global->monitor->box));
-
-    if (global->monitor->options.use_label)
-    {
-        gtk_widget_show(global->monitor->label);
-    }
-
-    if (global->monitor->options.show_values)
-    {
-        gtk_widget_modify_fg(global->monitor->rcv_label, GTK_STATE_NORMAL,
-                             (global->monitor->options.colorize_values ? &global->monitor->options.color[IN] : NULL));
-        gtk_widget_modify_fg(global->monitor->sent_label, GTK_STATE_NORMAL,
-                             (global->monitor->options.colorize_values ? &global->monitor->options.color[OUT] : NULL));
-        gtk_widget_show(global->monitor->rcv_label);
-        gtk_widget_show(global->monitor->sent_label);
     }
 
     if (!init_netload( &(global->monitor->data), global->monitor->options.network_device)
@@ -543,9 +577,10 @@ static void setup_monitor(t_global_monitor *global, gboolean supress_warnings)
         g_free(global->monitor->options.old_network_device);
     }
     global->monitor->options.old_network_device = g_strdup(global->monitor->options.network_device);
-    
-    run_update( global );
 
+    monitor_set_mode(global->plugin, xfce_panel_plugin_get_mode(global->plugin), global);
+
+    run_update( global );
 }
 
 
@@ -556,10 +591,10 @@ static void monitor_read_config(XfcePanelPlugin *plugin, t_global_monitor *globa
     char *file;
     XfceRc *rc;
     
-    if (!(file = xfce_panel_plugin_lookup_rc_file (plugin)))
+    if (!(file = xfce_panel_plugin_save_location (plugin, TRUE)))
         return;
     
-    rc = xfce_rc_simple_open (file, TRUE);
+    rc = xfce_rc_simple_open (file, FALSE);
     g_free (file);
 
     if (!rc)
@@ -669,31 +704,6 @@ static void monitor_write_config(XfcePanelPlugin *plugin, t_global_monitor *glob
 }
 
 
-/* ---------------------------------------------------------------------------------------------- */
-static gboolean monitor_set_size(XfcePanelPlugin *plugin, int size, t_global_monitor *global)
-{
-    gint i;
-
-    for (i = 0; i < SUM; i++)
-    {
-        if (xfce_panel_plugin_get_orientation (plugin) == GTK_ORIENTATION_HORIZONTAL)
-        {
-            gtk_widget_set_size_request(GTK_WIDGET(global->monitor->status[i]),
-                    BORDER, size - BORDER);
-        }
-        else
-        {
-            gtk_widget_set_size_request(GTK_WIDGET(global->monitor->status[i]),
-                    size - BORDER, BORDER);
-        }
-    }
-    PRINT_DBG("monitor_set_size");
-    setup_monitor(global, TRUE);
-
-    return TRUE;
-}
-
-
 /* ---------------------------------------------------------------------------------------------- */
 static void monitor_apply_options(t_global_monitor *global)
 {
@@ -1244,7 +1254,7 @@ static void netload_construct (XfcePanelPlugin *plugin)
     
     g_signal_connect (plugin, "size-changed", G_CALLBACK (monitor_set_size), global);
     
-    g_signal_connect (plugin, "orientation-changed", G_CALLBACK (monitor_set_orientation), global);
+    g_signal_connect (plugin, "mode-changed", G_CALLBACK (monitor_set_mode), global);
     
     gtk_container_add(GTK_CONTAINER(plugin), global->ebox);
     
